/*
 * Flashbook GUI
 * Copyright (c) 2008-2010 Benjamin Moody
 *
 * This program is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include <stdio.h>
#include <fb.h>
#include <gtk/gtk.h>
#include <glib/gstdio.h>
#include <errno.h>
#include "fbgui.h"
#include "fbpreview.h"

static GtkWidget *window, *preview;
static time_t lastloadtime = 0;
static char* fname;

static fbbook* read_book(gpointer data G_GNUC_UNUSED)
{
  FILE* f;
  fbbook* fb;

  f = g_fopen(fname, "rt");
  if (!f) {
    fprintf(stderr, "%s: cannot open %s: %s\n", g_get_prgname(),
	    fname, g_strerror(errno));
    return NULL;
  }

  fb = fb_book_new();
  fb->searchdir = fb_get_dirname(fname);

  if (fb_book_read_xml(fb, f, fname)) {
    fclose(f);
    fb_book_free(fb);
    return NULL;
  }
  fclose(f);

  fb_book_assign_font(fb);
  fb_chunk_convert_to_ti(fb, fb->body, 0);
  fb_chunk_split_lines(fb, fb->body);

  return fb;
}

static gboolean check_update(gpointer data G_GNUC_UNUSED)
{
  struct stat st;

  if (g_stat(fname, &st))
    return TRUE;

  if (st.st_mtime == lastloadtime)
    return TRUE;

  lastloadtime = st.st_mtime;
  fb_preview_refresh(FB_PREVIEW(preview));

  return TRUE;
}

int main(int argc, char** argv)
{
  gchar *p, *q;
  GtkWidget *box;
  GtkRequisition req;

  fix_main_args(&argc, &argv);
  gtk_init(&argc, &argv);

  if (argc != 2) {
    fprintf(stderr, "usage: %s [gtk-options] ebook-file\n", argv[0]);
    return 1;
  }
  fname = argv[1];

  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  box = fb_preview_box_new(read_book, NULL, &preview);
  gtk_container_add(GTK_CONTAINER(window), box);
  gtk_widget_show(box);

  gtk_widget_grab_focus(preview);

  gtk_widget_size_request(box, &req);
  gtk_window_set_default_size(GTK_WINDOW(window), -1, req.height + 128);

  g_signal_connect(window, "destroy", G_CALLBACK(gtk_main_quit), NULL);

  p = filename_to_display(argv[1]);
  q = g_strdup_printf("%s - Preview", p);
  gtk_window_set_title(GTK_WINDOW(window), q);
  g_free(p);
  g_free(q);

  g_timeout_add(200, check_update, NULL);
  check_update(NULL);

  gtk_widget_show(window);
  gtk_main();
  return 0;
}
