/*
 * Flashbook Font Editor
 * Copyright (c) 2008-2010 Benjamin Moody
 *
 * This program is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#define FB_MAX_FONT_HEIGHT 16

typedef struct _editinfo {
  struct _appinfo *app;

  gchar* filename;
  gboolean modified;
  gboolean active;
  gboolean autometric;

  gchar* headers;
  int charset;
  int cell_width;
  int cell_height;
  int min_row_height;
  int baseline;
  int default_width;
  GTree* bitmap_tree;
  fbfontchar* current_bitmap;
  fbfontchar tmp_bitmap;
  gboolean current_modified;

  GtkWidget* box;
  GtkWidget* cb_builtin;
  GtkWidget* cb_allowover;
  GtkWidget* btn_delete;
  GtkWidget* sb_width;
  GtkWidget* da_edit;
  GtkWidget* da_preview;
  int mousebutton;
  int kbeditx, kbedity;
} editinfo;

typedef struct _appinfo {
  GtkWidget* mainwin;
  GtkWidget* charmap;
  GtkWidget* editbox;
  GtkAdjustment* edithadj;
  GList* edits;
  editinfo* active_edit;

  gchar* preview_left_text;
  gchar* preview_right_text;
} appinfo;

fbfontchar* editor_add_character(editinfo* ei, const fbfontchar* bm);

gboolean find_builtin_bitmap(editinfo* ei, const fbfontchar* bm);
gboolean load_builtin_bitmap(editinfo* ei, fbfontchar* bm);
void load_builtin_bitmaps(editinfo* ei);

/* Unified MDI functions */
void fontedit_init(gboolean (*focusin)(), gboolean (*focusout)());
void fontedit_new();
void fontedit_open(const char* fname);
void fontedit_save();
void fontedit_save_as();
void fontedit_close();
gboolean fontedit_prompt_save();
void fontedit_quit();

