/*
 * libflashbook - Library for creating eBooks for the TI-83/84 Plus
 * Copyright (c) 2008 Benjamin Moody
 *
 * This program is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include "fb.h"

int fb_error(fbbook* fb, int errcode, const char* fmt, ...)
{
  va_list ap;
  char* s;
  unsigned len = 255;

  if (!fb || !fb->errfunc)
    return errcode;

  s = fbnew(char, len + 1);
  while (1) {
    s[len] = 0;
    va_start(ap, fmt);
    vsnprintf(s, len, fmt, ap);
    va_end(ap);
    if (strlen(s) < len - 1) {
      (*fb->errfunc)(fb, errcode, s);
      fbfree(s);
      return errcode;
    }
    len = (len * 2) + 1;
    s = fbrenew(char, s, len + 1);
  }
}

void fb_error_default(fbbook* fb FBATTR_UNUSED, int errcode FBATTR_UNUSED,
                      const char* message)
{
  if (errcode)
    fputs("Error: ", stderr);
  fprintf(stderr, "%s\n", message);
}
